// TODO: weekly quest => summary banner
const BEGIN='2021-01-20';
const MAX_TAGS=20;

function _add(list, item) {
	if (list.indexOf(item)===-1)
		list.push(item);
}
// remove all edges with weight < n
// remove all vertices with edges count < n
function purge_graph(g, n) {
	g=JSON.parse(JSON.stringify(g));
	for (let i in g) {
		for (let j in g) {
			if (g[i][j]!==undefined && g[i][j]<n) {
				delete g[i][j];
				delete g[j][i];
			}
		}
	}

	let to_delete=[];
	for (let i in g)
		if (Object.keys(g[i]).length<n)
			to_delete.push(i);

	to_delete.forEach(i => {
		delete g[i];
	});

	return g;
}


var wdb={};
var rdb;
function compute() {
	_compute_global();
	db.forEach(dream => {
		let week_no=get_week_no(dream.date)
		if (!(week_no in wdb))
			wdb[week_no]={
				dreams: [],
				authors: {},
				tags: {},
				quest_achievers: [],
				qdreams_count: 0,
				nqdreams_count: 0
			}

		wdb[week_no].dreams.push(dream);

		if (!(dream.author in wdb[week_no].authors))
			wdb[week_no].authors[dream.author]={
				dreams_count: 0
			}

		wdb[week_no].authors[dream.author].dreams_count++;

		dream.tags.forEach(tag => {
			if (!(tag in wdb[week_no].tags)) {
				wdb[week_no].tags[tag]={
					dreams_count: 0
				}
			}
			wdb[week_no].tags[tag].dreams_count++;

			if (week_no in quests) {
				if ("tags" in quests[week_no]
						&& quests[week_no].tags.indexOf(tag)!==-1) {
					// quest hit
					if (wdb[week_no].quest_achievers.indexOf(
							dream.author)===-1)
						wdb[week_no].quest_achievers.push(dream.author);

					wdb[week_no].qdreams_count++;
				} else {
					wdb[week_no].nqdreams_count++;
				}
			}
		});
	});

	for (let week_no in wdb) {
		wdb[week_no].clique=compute_clique(wdb[week_no].dreams);
		wdb[week_no].rating=wdb[week_no].clique.length;
	}
	rdb=JSON.parse(JSON.stringify(db));
	rdb.reverse();
}
function get_clique(v, g, n) {
	g=JSON.parse(JSON.stringify(g));

	if (n===1) {
		if (Object.keys(g).length===0)
			throw Error("wtf?!");
		return [Object.keys(g)[0]];
	}

	let to_delete=[];
	for (let i in g)
		if (i!==v && g[v][i]===undefined)
			to_delete.push(i);

	to_delete.forEach(i => {
		delete g[i];
	});

	delete g[v];

	for (let i in g) {
		let clique=get_clique(i, g, n-1);
		if (clique!==undefined) {
			clique.unshift(v);
			return clique;
		}
	}

	return undefined;
}

function build_graph(dreams) {
	let g={};

	for (let i=0; i<dreams.length; i++)
		for (let j=0; j<dreams.length; j++) {
			if (i==j)
				continue;

			let shared_tags_count=dreams[i].tags.filter(tag =>
				dreams[j].tags.indexOf(tag)!==-1).length;

			if (shared_tags_count === 0)
				continue;

			if (!(i in g)) g[i]={};
			if (!(j in g)) g[j]={};

			g[i][j]=g[j][i]=shared_tags_count;
		}
	return g;
}


let _clique_cache={};
function compute_clique(dreams) {
	let cache_key=JSON.stringify(dreams);
	if (cache_key in _clique_cache)
		return _clique_cache[cache_key];
	let Graph=build_graph(dreams);

	let last_clique;
	for (let n=1; n<dreams.length; n++) {
		let graph=purge_graph(Graph, n);
		
		let found=false;

		for (let i in graph) {
			clique=get_clique(i, graph, n);
			if (clique!==undefined) {
				found=true;
				last_clique=clique;
				break;
			}
		}

		if (!found) {
			_clique_cache[cache_key]=last_clique;
			return last_clique;
		}
	}

	_clique_cache[cache_key]=Object.keys(dreams);
	return Object.keys(dreams);
}

function go_quests() {
	if (back_flag)
		history.replaceState({ page: "quests"}, "quests");
	else
		history.pushState({ page: "quests"}, "quests");
	back_flag=false;

	change_active_menu_item("quests_btn");

	let content=_el("div", undefined, "quests_list");
	content.appendChild(_el("div", "Квесты", "header"));

	Object.keys(quests).sort((a, b)=>parseInt(b)-parseInt(a))
			.forEach(week_no => {
		let div=_el("div");
		div.appendChild(span(`Неделя ${week_no}: `));
		quests[week_no].tags.forEach(tag => {
			div.appendChild(create_tag(tag));
			div.appendChild(span(' '));
		});
		content.appendChild(div);
	});

	replaceContent(content);
}

function go_users(filter, caption) {
	let fkey=null, ckey=null;
	if (filter!==undefined) {
		_filters.push(filter);
		fkey=_filters.length-1;
	}
	if (caption!==undefined) {
		_captions.push(caption);
		ckey=_captions.length-1;
	}
	if (back_flag)
		history.replaceState({ page: "users", args: {fkey, ckey}},
			"users", "");
	else
		history.pushState({ page: "users", args: {fkey, ckey}},
			"users", "");
	back_flag=false;

	change_active_menu_item("users_btn");

	let list=_el("div", undefined, "users_list");

	let n=1;

	if (filter===undefined)
		filter=()=>true;

	if (caption===undefined)
		caption="Все юзеры";

	if (typeof caption==="string")
		caption=_el("div", caption, "header");

	list.appendChild(caption);

	Object.keys(_authors).filter(filter).sort((a, b) => {
		if (_authors[a].rating!==_authors[b].rating)
			return _authors[b].rating-_authors[a].rating;
		else
			return _authors[b].dreams_count
				- _authors[a].dreams_count;
	}).forEach(author => {
		let div=_el("div", undefined, "text");
		div.appendChild(span(`${n++}. `));
		div.appendChild(create_author_banner(author));
		div.appendChild(span(" ("));
		div.appendChild(span("рейтинг", "link", ()=>{ go_info(); }));
		div.appendChild(span(`: ${_authors[author].rating}, снов: `));
		div.appendChild(span(_authors[author].dreams_count, "link", ()=>{
			let caption=create_author_banner_caption(author);
			go_dreams(dream => dream.author===author, caption);
		}));
		div.appendChild(span(")"));

		list.appendChild(div);
	});

	replaceContent(list);
}

var _after_replace_content=[];

function create_author_banner_caption(author) {
	let caption=_el("div", undefined, "header");
	caption.appendChild(span(`Сны #${author} (`));
	caption.appendChild(span('рейтинг юзера', 'link'));
	caption.appendChild(span(`: ${_authors[author].rating})`));
	caption.appendChild(_el("br"));
	let canvas=_el("canvas");
	caption.appendChild(canvas);
	_after_replace_content.push(()=>{
		dreams_graph(canvas, rdb.filter(dream => dream.author===author));
	});
	return caption;
}

function dreams_graph(canvas, dreams) {
	let el=canvas;

	let min_date;

	for (let i in dreams) {
		let dream=dreams[i];
		if (!min_date || min_date>dream.date)
			min_date=dream.date;
	}
	min_date='2021-01-20';
	min_date=new Date(min_date)

	let days=[];
	let legend=[];
	for (let i in dreams) {
		let dream=dreams[i];
		let d=(new Date(dream.date)-min_date)/1000/60/60/24;
		while (days.length<=d) {
			//legend.push(days.length);
			let cur=add_days(min_date, days.length);
			let l=cur.toISOString()
				.substr(8, 2);

			if (l==='01' || days.length===0) {
				l+='.' + cur.toISOString().substr(5, 2);
			}
			days.push(0);
			legend.push(days.length%2===1? l: '');
		}
		days[d]++;
	}

	let max_date=get_last_dream_date();

	while (true) {
		let cur=add_days(min_date, days.length);
		if (cur>max_date)
			break;

		let l=cur.toISOString().substr(8, 2);
		if (l==='01' || days.length===0)
			l+='.' + cur.toISOString().substr(5, 2);

		days.push(0);
		legend.push(l);
	}

	let chart=new Graph({
		data: days,
		target: el,
    // Internal padding for the canvas
    paddingBottom:  0, 
    paddingLeft:    0,
    paddingRight:   0,
    paddingTop:     30,

    // Canvas background
    background:     "#FFFFFFFF",

    // Show circle on the last data
    showCircle:     false,
    circle:         "#55AA77",
    circleSize:     4,

    // Show the line at zero
    centerZero:     false,
    showZeroLine:   true,
    zeroLineColor:  "#EEE",

    // Color and whidth of the line chart
    lineColor:      "#8888EE",
    lineWidth:      3,

    // Show the bounds
    showBounds:     true,
    bounds:         "#8888EE",
    boundsHeight:   14,
    boundsFont:     "Arial",

    // Show the legend
    showLegend:     true,
    legend:         legend,
    legendHeight:   14,
    legendFont:     "Arial",
	});
}


function go_tags(filter, caption) {
	let fkey=null, ckey=null;
	if (filter!==undefined) {
		_filters.push(filter);
		fkey=_filters.length-1;
	}
	if (caption!==undefined) {
		_captions.push(caption);
		ckey=_captions.length-1;
	}
	if (back_flag)
		history.replaceState({ page: "tags", args: {fkey, ckey}},
			"tags", "");
	else
		history.pushState({ page: "tags", args: {fkey, ckey}},
			"tags", "");
	back_flag=false;

	change_active_menu_item("tags_btn");

	let list=_el("div", undefined, "tags_list");

	let n=1;

	if (filter===undefined)
		filter=()=>true;

	if (caption===undefined)
		caption="Все теги";

	if (typeof caption==="string")
		caption=_el("div", caption, "header");

	list.appendChild(caption);

	Object.keys(_tags).filter(filter).sort((a, b) => {
		if (_tags[a].rating!==_tags[b].rating)
			return _tags[b].rating-_tags[a].rating;
		else
			return _tags[b].dreams_count
				- _tags[a].dreams_count;
	}).forEach(tag => {
		let div=_el("div", undefined, "text");
		div.appendChild(span(`${n++}. `));
		div.appendChild(create_tag(tag));
		div.appendChild(span(" ("));
		div.appendChild(span("рейтинг", "link", ()=>{ go_info(); }));
		div.appendChild(span(`: ${_tags[tag].rating}, снов: `));
		div.appendChild(span(_tags[tag].count, "link", ()=>{
			let caption=span(undefined, "header");
			caption.appendChild(span(`Сны с тегом #${tag} (`));
			caption.appendChild(span('рейтинг тега', 'link'));
			caption.appendChild(span(`: ${_tags[tag].rating})`));
			go_dreams(dream => dream.tags && dream.tags.indexOf(tag)!==-1,
				caption);
		}));
		div.appendChild(span(")"));

		list.appendChild(div);
	});

	replaceContent(list);
}

function main() {
	onpopstate=event => {
		let args=event.state.args;
		back_flag=true;
		switch (event.state.page) {
			case "dreams":
				return go_dreams(
					args.fkey===null? undefined: _filters[args.fkey],
					args.ckey===null? undefined: _captions[args.ckey],
					args.key);
			case "users":
				return go_users(
					args.fkey===null? undefined: _filters[args.fkey],
					args.ckey===null? undefined: _captions[args.ckey]);
			case "tags":
				return go_tags(
					args.fkey===null? undefined: _filters[args.fkey],
					args.ckey===null? undefined: _captions[args.ckey]);
			case "home":
				return go_home();
			case "quests":
				return go_quests();
			case "info":
				return go_info();
			case "graphs":
				return go_graphs();
			default:
				console.error(event);
		}
	};
	compute();
	go_home();
}

function get_week_start(week_no) {
	let wed=new Date(BEGIN).valueOf()+(week_no-1)*7*24*60*60*1000;
	return new Date(wed).toISOString().substr(0, 10);
}

var _tape_cache={};
var _filters=[];
var _captions=[];
let back_flag=false;
function go_dreams(filter, caption, key) {
	let fkey=null, ckey=null;
	if (filter!==undefined) {
		_filters.push(filter);
		fkey=_filters.length-1;
	}
	if (caption!==undefined) {
		_captions.push(caption);
		ckey=_captions.length-1;
	}
	if (back_flag)
		history.replaceState({ page: "dreams", args: {fkey, ckey, key}},
			"dreams", "");
	else
		history.pushState({ page: "dreams", args: {fkey, ckey, key}},
			"dreams", "");
	back_flag=false;

	change_active_menu_item("dreams_btn");

	if (key!==undefined && key in _tape_cache)
		return replaceContent(_tape_cache[key]);

	let content=_el("div", undefined, "dreams_tape")

	let tape=_el("div", undefined, "body");
	rdb.forEach(dream => {
		if (filter===undefined || filter(dream))
			tape.appendChild(dream_banner(dream));
	});

	if (caption===undefined)
		caption="Все сны";

	if (typeof caption==="string")
		caption=span(caption, "header");

	content.appendChild(caption);
	content.appendChild(tape);

	if (key!==undefined)
		_tape_cache[key]=content;

	replaceContent(content);
}

var _dream_banner_cache={};
function dream_banner(dream) {
	let key=JSON.stringify(dream);
	if (!(key in _dream_banner_cache)) {
		let banner=_el("div", undefined, "dream_banner");

		banner.appendChild(create_date(dream.date));
		banner.appendChild(span(' '));
		banner.appendChild(create_author_banner(dream.author));
		banner.appendChild(span(' ('));
		banner.appendChild(span(`рейтинг сна`, "link", ()=>{
			go_info();
		}));
		banner.appendChild(span(`: ${dream.rating}), источник: `));
		let a=_el("a", dream.pid, "source_ref");
		a.href=`https://dreamhackers.eu/viewtopic.php?p=${dream.pid}#p${dream.pid}`;
		a.target="_blank";
		banner.appendChild(a);
		let div=_el("div", undefined, "dream_body");
		div.innerHTML=dream.html;
		banner.appendChild(div);

		dream.tags.forEach(tag => {
			banner.appendChild(create_tag(tag));
			banner.appendChild(span(' '));
		});

		_dream_banner_cache[key]=banner;
	}

	return _dream_banner_cache[key];
}

function create_tag(tag) {
	let el=_el("span", '#'+tag, "tag_banner", ()=>{
		let caption=span(undefined, "header");
		caption.appendChild(span(`Сны с тегом #${tag} (`));
		caption.appendChild(span('рейтинг тега', 'link', go_info));
		caption.appendChild(span(`: ${_tags[tag].rating})`));
		go_dreams(dream => dream.tags
			&& dream.tags.indexOf(tag)!==-1,
			caption);
	});
	return el;
}

function create_date(date) {
	if (typeof date==="string")
		date=new Date(date);

	let sdate=date.toISOString().substr(0, 10);

	return span(sdate, "link", ()=>{
		go_dreams(dream => dream.date===sdate,
			`Сны ${date.toISOString().substr(0, 10)}`);
	});
}

function create_author_banner(author) {
	let el=_el("span", author, "author_banner", ()=>{

		let caption=create_author_banner_caption(author);
		go_dreams(dream => dream.author===author, caption);
	});
	return el;
}

function change_active_menu_item(item_id) {
	document.body.querySelector("#top_menu")
		.querySelectorAll(".menu_button").forEach(btn => {
			btn.className="menu_button";
		});
	document.body.querySelector("#top_menu").querySelector("#"+item_id)
		.className="menu_button menu_button_active";
}
function add_days(date, days) {
	return new Date(date.valueOf() + days*1000*60*60*24);
}


let _next_content, _replacing=false;
function replaceContent(el) {
	if (_replacing) {
		return _next_content=el;
	}

	_replacing=true;
	_next_content=undefined;

	let target=document.body.querySelector("#content");

	let loading=span("Загрузка...", "loading");
	target.innerHTML='';
	target.appendChild(loading);
	setTimeout(()=>{
		target.appendChild(el);
		target.removeChild(loading);

		while (_after_replace_content.length>0) {
			_after_replace_content.pop()();
		}

		_replacing=false;
		if (_next_content!==undefined)
			replaceContent(_next_content);
	}, 1);
}

function go_info() {
	if (back_flag)
		history.replaceState({ page: "info" }, "info", "");
	else
		history.pushState({ page: "info" }, "info", "");
	back_flag=false;

	change_active_menu_item("info_btn");

	let info=_el("div");
	info.innerHTML=document.body.querySelector("#info_body").innerHTML;
	replaceContent(info);
}

function go_graphs() {
	if (back_flag)
		history.replaceState({ page: "graphs" }, "graphs", "");
	else
		history.pushState({ page: "graphs" }, "graphs", "");
	back_flag=false;

	change_active_menu_item("graphs_btn");

	let graphs=_el("div");
	graphs.innerHTML=document.body.querySelector("#graphs_body").innerHTML;
	replaceContent(graphs);
}

var home_el;
function go_home() {
	if (back_flag)
		history.replaceState({ page: "home" }, "home", "");
	else
		history.pushState({ page: "home" }, "home", "");
	back_flag=false;

	change_active_menu_item("home_btn");

	if (home_el===undefined) {
		home_el=document.createElement("div");

		home_el.appendChild(group_banner());
		
		for(let i=get_last_week_no(); i>=1; i--)
			home_el.appendChild(week_summary_banner(i));

		_toggle_week_banner[get_last_week_no()]();
	}

	replaceContent(home_el);
}


function group_banner() {
	let el=_el("div", undefined, "group_banner");
	el.appendChild(span("Рейтинг группы", "link", ()=>{go_info();}));
	el.appendChild(span(`: ${_group_rating}; юзеров: `));
	el.appendChild(span(Object.keys(_authors).length, "link", ()=>{
		go_users();}));
	el.appendChild(span(", снов: "));
	el.appendChild(span(db.length, "link", ()=>{go_dreams();}));
	el.appendChild(span(", тегов: "));
	el.appendChild(span(Object.keys(_tags).length, "link", ()=>{
		go_tags();}));
	return el;
}

var _week_summary_banners={};
var _toggle_week_banner={};
function week_summary_banner(week_no) {
	if (!(week_no in _week_summary_banners)) {
		let week=wdb[week_no];
		let banner=_el("div", undefined, "week_summary_banner");
		let header=_el("div", undefined, "header");
		header.appendChild(span(`Неделя ${week_no}. Юзеров: `));
		header.appendChild(span(Object.keys(week.authors).length,
				"link", () => {
			go_users(author => author in week.authors,
		nn		`Юзеры, сновидящие на неделе ${week_no}`);
			_toggle_week_banner[week_no]();
		}));

		header.appendChild(span(", снов: "));
		header.appendChild(span(week.dreams.length,
				"link", ()=>{
			go_dreams(dream => dream.date>=get_week_start(week_no)
				&& dream.date<get_week_start(week_no+1),
				`Сны на неделе ${week_no}`);
			_toggle_week_banner[week_no]();
		}));

		header.appendChild(span(", тегов: "));
		header.appendChild(span(Object.keys(week.tags).length,
				"link", ()=>{
			go_tags(tag => tag in week.tags,
				`Теги на неделе ${week_no}`);
			_toggle_week_banner[week_no]();
		}));

		header.appendChild(span(", "));
		header.appendChild(span("синхро-индекс", "link", ()=>{
			go_info();
			_toggle_week_banner[week_no]();
		}));
		header.appendChild(span(`: ${week.rating}`));

		if (week_no in quests) {
			header.appendChild(span(", "));
			header.appendChild(span("задание", "link", ()=>{
				go_quests();
				_toggle_week_banner[week_no]();
			}));
			header.appendChild(span(" выполнили: "));
			header.appendChild(span(week.quest_achievers.length,
					"link", ()=>{
				go_users(author => 
					week.quest_achievers.indexOf(author)!==-1,
					`Юзеры, выполнившие задание на неделе ${week_no}`);
				_toggle_week_banner[week_no]();
			}));
			header.appendChild(span(` (${parseInt(100*week.qdreams_count
				/ (week.qdreams_count+week.nqdreams_count))}% снов)`));
		}

		banner.appendChild(header);

		let body=_el("div", undefined, "body");
		let num=week_no-1<10? '0'+(week_no-1): ''+(week_no-1);
		body.appendChild(get_img(`data/generated/wcloud_${num}.png`));

		body.appendChild(_el("br"));

		let flag;
		if (week_no in quests) {
			flag=1;
			if (week.quest_achievers.length>0)
				body.appendChild(span("Задание выполнили: "));
			else
				body.appendChild(span("Задание не выполнили: "));
		} else
			body.appendChild(span("Юзеры: "));
		Object.keys(week.authors).sort((a, b) => {
			if (week_no in quests
					&& week.quest_achievers.indexOf(a)===-1
					&& week.quest_achievers.indexOf(b)!==-1) {
				return 1;
			}

			if (week_no in quests
					&& week.quest_achievers.indexOf(a)!==-1
					&& week.quest_achievers.indexOf(b)===-1) {
				return -1;
			}

			return week.authors[b].dreams_count
				- week.authors[a].dreams_count;
		}).forEach(author => {
			let qa=week.quest_achievers.indexOf(author)!==-1;
			if (flag===1) {
				flag=2;
			} else if (flag===2 && !qa) {
				flag=3;
				body.appendChild(span("; не выполнили: "));
			} else
				body.appendChild(span(", "));
			body.appendChild(create_author_banner(author));
			body.appendChild(span(" ("));
			body.appendChild(span(`${week.authors[author].dreams_count}`,
					"link", ()=>{
				go_dreams(dream => dream.author===author
					&& dream.date>=get_week_start(week_no)
					&& dream.date<get_week_start(week_no+1),
					`Сны ${author} на неделе ${week_no}`);
			}));
			body.appendChild(span(")"));
		});

		body.appendChild(_el("br"));

		body.appendChild(span("Теги: "));
		
		// tags sorted by occurances' count
		Object.keys(week.tags).sort((a, b) =>
				week.tags[b].dreams_count -
				week.tags[a].dreams_count).slice(0, MAX_TAGS)
				.forEach(tag => {
			body.appendChild(create_tag(tag));
			body.appendChild(span(' ('));
			body.appendChild(span(week.tags[tag].dreams_count, "link",
					()=>{
				go_dreams(dream => dream.tags.indexOf(tag)!==-1
					&& dream.date>=get_week_start(week_no)
					&& dream.date<get_week_start(week_no+1),
					`Сны с тегом #${tag} на ${week_no} неделе`);
			}));
			body.appendChild(span(") "));
		});

		if (Object.keys(week.tags).length>MAX_TAGS) {
			body.appendChild(span(`...все (${
					Object.keys(week.tags).length})`, "link", ()=>{
				go_tags(tag => tag in week.tags);
			}));
		}

		body.style.display="none";

		banner.appendChild(body);

		header.onclick=_toggle_week_banner[week_no]=()=>{
			body.style.display=body.style.display==="none"?
				"block": "none";
		};

		_week_summary_banners[week_no]=banner;
	}

	return _week_summary_banners[week_no];
}

var _img_cache={};
function get_img(path) {
	if (!(path in _img_cache)) {
		let img=document.createElement("img");
		img.setAttribute("src", path);
		_img_cache[path]=img;
	}

	return _img_cache[path];
}


function span(text, class_name, onclick) { 
	return _el("span", text, class_name, onclick);
}

function _el(el_name, text, class_name, onclick) {
	let el=document.createElement(el_name);
	if (text!==undefined)
		el.innerText=text;
	if (class_name!==undefined)
		el.className=class_name;
	if (onclick!==undefined)
		el.onclick=onclick;
	return el;
}

const MAX=(x, y)=>(x>y? x: y);
const MIN=(x, y)=>(x<y? x: y);

var _last_dream_date;
function get_last_dream_date() {
	if (_last_dream_date===undefined) {
		_last_dream_date=db.map(d => d.date).reduce(MAX);

		if (_last_dream_date!==undefined)
			_last_dream_date=new Date(_last_dream_date);
	}

	return _last_dream_date;
}

function get_last_week_no() {
	return get_week_no(get_last_dream_date());
}

function get_week_no(date) {
	if (typeof date==="string")
		date=new Date(date);

	return Math.floor((date.valueOf()
		-new Date(BEGIN).valueOf()) /1000/60/60/24/7 + 1);
}

function _compute_global() {
	_tags={};
	_authors={};

	db.forEach(dream => {
		if (dream.tags) dream.tags.forEach(tag => {
			if (!(tag in _tags))
				_tags[tag]={
					count: 0,
					authors: {}
				};

			_tags[tag].count++;
			if (!(dream.author in _tags[tag].authors))
				_tags[tag].authors[dream.author]=0;

			_tags[tag].authors[dream.author]++;
		});

		if (!(dream.author in _authors))
			_authors[dream.author]={
				dreams_count: 0,
				rated_count: {},
				rating: 0
			};

		_authors[dream.author].dreams_count++;
	});

	// compute tags rating: n authors used tag n times
	for (let tag in _tags) {
		let n=Object.keys(_tags[tag].authors).length;

		let top_authors=_get_top_tag_authors(tag);

		while (true) {
			if (_tags[tag].authors[top_authors[n-1]]>=n) {
				_tags[tag].rating=n;
				break;
			}

			n--;
		}
	}

	// compute dreams rating: n tags with rating >= n
	// and compute authors rating: n dreams with rating >= n
	db.forEach(dream => {
		let top_tags=dream.tags.sort((a, b) => {
			return _tags[b].rating - _tags[a].rating;
		});

		let n=top_tags.length;
		dream.rating=0;

		while (n>0) {
			if (_tags[top_tags[n-1]].rating>=n) {
				dream.rating=n;
				
				if (!(n in _authors[dream.author].rated_count))
					_authors[dream.author].rated_count[n]=0;

				_authors[dream.author].rated_count[n]++;

				if (_authors[dream.author].rated_count[n]>=n
						&& _authors[dream.author].rating<n)
					_authors[dream.author].rating=n;
				break;
			}

			n--;
		}
	});

	// compute group rating
	let top_authors=Object.keys(_authors).sort((a, b) => {
		return _authors[b].rating-_authors[a].rating;
	});

	for (let n=top_authors.length; n>=0; n--)
		if (_authors[top_authors[n-1]].rating>=n) {
			_group_rating=n;
			break;
		}
}


function _get_top_tag_authors(tag) {
	return Object.keys(_tags[tag].authors).sort((a, b) => {
		return _tags[tag].authors[b]-_tags[tag].authors[a];
	});
}

