const BEGIN='2021-01-20';
const MAX_TAGS=20;

const min=(x, y)=>x<y? x: y;
const max=(x, y)=>x>y? x: y;

function componentToHex(c) {
  var hex = parseInt(c).toString(16);
  return hex.length == 1 ? "0" + hex : hex;
}

function rgbToHex(r, g, b) {
  return "#" + componentToHex(r) + componentToHex(g) + componentToHex(b);
}


function _ratings() {
	let dreams=db.map(dream => dream.rating);
	console.log('DREAMS:', dreams.reduce(min), dreams.reduce(max));

	let tags=Object.keys(_tags).map(tag => _tags[tag].rating);
	let i0, i1
	console.log('TAGS:', i0=tags.reduce(min), i1=tags.reduce(max));

	let authors=Object.keys(_authors).map(author =>
		_authors[author].rating);
	console.log('AUTHORS:', authors.reduce(min), authors.reduce(max));

	let styles='';
	for (let i=i0; i<=i1; i++) {
		let k=(i-i0)*1.0/(i1-i0);
		styles+=`
.tag_level_${i} { color: ${rgbToHex(0, 128-k*128, 128*k)} }`;
	}
	console.log(styles);

}

let _img_cache={};
function get_img(path) {
	if (!(path in _img_cache)) {
		let img=document.createElement("img");
		img.setAttribute("src", path);
		_img_cache[path]=img;
	}

	return _img_cache[path];
}

let _clique_cache={};
function compute_clique(dreams) {
	let cache_key=JSON.stringify(dreams);
	if (cache_key in _clique_cache)
		return _clique_cache[cache_key];
	let Graph=build_graph(dreams);

	let last_clique;
	for (let n=1; n<dreams.length; n++) {
		let graph=purge_graph(Graph, n);
		
		let found=false;

		for (let i in graph) {
			clique=get_clique(i, graph, n);
			if (clique!==undefined) {
				found=true;
				last_clique=clique;
				break;
			}
		}

		if (!found) {
			_clique_cache[cache_key]=last_clique;
			return last_clique;
		}
	}

	_clique_cache[cache_key]=Object.keys(dreams);
	return Object.keys(dreams);
}

function get_clique(v, g, n) {
	g=JSON.parse(JSON.stringify(g));

	if (n===1) {
		if (Object.keys(g).length===0)
			throw Error("wtf?!");
		return [Object.keys(g)[0]];
	}

	let to_delete=[];
	for (let i in g)
		if (i!==v && g[v][i]===undefined)
			to_delete.push(i);

	to_delete.forEach(i => {
		delete g[i];
	});

	delete g[v];

	for (let i in g) {
		let clique=get_clique(i, g, n-1);
		if (clique!==undefined) {
			clique.unshift(v);
			return clique;
		}
	}

	return undefined;
}

// remove all edges with weight < n
// remove all vertices with edges count < n
function purge_graph(g, n) {
	g=JSON.parse(JSON.stringify(g));
	for (let i in g) {
		for (let j in g) {
			if (g[i][j]!==undefined && g[i][j]<n) {
				delete g[i][j];
				delete g[j][i];
			}
		}
	}

	let to_delete=[];
	for (let i in g)
		if (Object.keys(g[i]).length<n)
			to_delete.push(i);

	to_delete.forEach(i => {
		delete g[i];
	});

	return g;
}

function build_graph(dreams) {
	let g={};

	for (let i=0; i<dreams.length; i++)
		for (let j=0; j<dreams.length; j++) {
			if (i==j)
				continue;

			let shared_tags_count=dreams[i].tags.filter(tag =>
				dreams[j].tags.indexOf(tag)!==-1).length;

			if (shared_tags_count === 0)
				continue;

			if (!(i in g)) g[i]={};
			if (!(j in g)) g[j]={};

			g[i][j]=g[j][i]=shared_tags_count;
		}
	return g;
}

function show_global_stats() {
	clear(week_view.el);
	let back_div=document.createElement("div");
	back_div.appendChild(_button("Назад", ()=>{
		week_view.el.removeChild(back_div);
		week_view.week_header.style.display="block";
		week_view._reset();
	}));
	back_div.appendChild(_t(" Графики"));
	week_view.el.appendChild(back_div);

	week_view.el.appendChild(_br());

	let img=document.createElement("img");
	img.setAttribute("src", "data/generated/cloud.png");
	img.style.width="100%"
	week_view.el.appendChild(img);
	week_view.el.appendChild(_br());


	img=document.createElement("img");
	img.setAttribute("src", "data/generated/dpd.png");
	img.style.width="100%"
	week_view.el.appendChild(img);
	week_view.el.appendChild(_br());
	week_view.el.appendChild(_br());

	img=document.createElement("img");
	img.setAttribute("src", "data/generated/ttpd.png");
	img.style.width="100%"
	week_view.el.appendChild(img);
	week_view.el.appendChild(_br());
	week_view.el.appendChild(_br());



	// dreams per day
	// total tags [begin; date]
}

function show_all_dreams() {
	// Show all dreams
	clear(week_view.el);

	let back_div=document.createElement("div");
	back_div.appendChild(_button("Назад", ()=>{
		week_view.el.removeChild(back_div);
		week_view.week_header.style.display="block";
		week_view._reset();
	}));
	back_div.appendChild(_t(" Просмотр ВСЕХ снов"));
	week_view.el.appendChild(back_div);

	week_view._tape=new DreamsTape(() => true);
	week_view.el.appendChild(week_view._tape.el);
	scrollTo(0, 0);
}

function show_all_authors() {
	clear(week_view.el);

	let back_div=document.createElement("div");
	back_div.appendChild(_button("Назад", ()=>{
		week_view.el.removeChild(back_div);
		week_view.week_header.style.display="block";
		week_view._reset();
	}));
	back_div.appendChild(_t(" Просмотр авторов за всё время"));
	week_view.el.appendChild(back_div);

	let sorted_authors=Object.keys(_authors).sort((a, b) => {
		return _authors[b].dreams_count - _authors[a].dreams_count;
	});

	for (let i=0; i<sorted_authors.length; i++) {
		let author=sorted_authors[i];
		week_view.el.appendChild(author_banner(author));
		week_view.el.appendChild(_t(` (${_authors[author].dreams_count})`));
		week_view.el.appendChild(_br());
	}
	scrollTo(0, 0);
	scrollTo(0, 0);
}

function show_all_tags() {
	clear(week_view.el);

	let back_div=document.createElement("div");
	back_div.appendChild(_button("Назад", ()=>{
		week_view.el.removeChild(back_div);
		week_view.week_header.style.display="block";
		week_view._reset();
	}));
	back_div.appendChild(_t(" Просмотр тегов за всё время"));
	week_view.el.appendChild(back_div);

	let sorted_tags=Object.keys(_tags).sort((a, b) => {
		return _tags[b].count - _tags[a].count;
	});

	for (let i=0; i<sorted_tags.length; i++) {
		let tag=sorted_tags[i];
		week_view.el.appendChild(tag_banner(tag));
		week_view.el.appendChild(_t(` (${_tags[tag].count})`));
		week_view.el.appendChild(_br());
	}
	scrollTo(0, 0);
	scrollTo(0, 0);
}

function clear(el) {
	while (el.children.length>0)
		el.removeChild(el.children[0]);
	el.innerHTML='';
}

var _tags, _authors, _group_rating;
function get_tags_count() {
	if (_tags===undefined)
		_compute_global();

	return Object.keys(_tags).length;
}

function get_authors_count() {
	if (_authors===undefined)
		_compute_global();

	return Object.keys(_authors).length;
}

function _get_top_tag_authors(tag) {
	return Object.keys(_tags[tag].authors).sort((a, b) => {
		return _tags[tag].authors[b]-_tags[tag].authors[a];
	});
}

function _compute_global() {
	_tags={};
	_authors={};

	db.forEach(dream => {
		if (dream.tags) dream.tags.forEach(tag => {
			if (!(tag in _tags))
				_tags[tag]={
					count: 0,
					authors: {}
				};

			_tags[tag].count++;
			if (!(dream.author in _tags[tag].authors))
				_tags[tag].authors[dream.author]=0;

			_tags[tag].authors[dream.author]++;
		});

		if (!(dream.author in _authors))
			_authors[dream.author]={
				dreams_count: 0,
				rated_count: {},
				rating: 0
			};

		_authors[dream.author].dreams_count++;
	});

	// compute tags rating: n authors used tag n times
	for (let tag in _tags) {
		let n=Object.keys(_tags[tag].authors).length;

		let top_authors=_get_top_tag_authors(tag);

		while (true) {
			if (_tags[tag].authors[top_authors[n-1]]>=n) {
				_tags[tag].rating=n;
				break;
			}

			n--;
		}
	}

	// compute dreams rating: n tags with rating >= n
	// and compute authors rating: n dreams with rating >= n
	db.forEach(dream => {
		let top_tags=dream.tags.sort((a, b) => {
			return _tags[b].rating - _tags[a].rating;
		});

		let n=top_tags.length;
		dream.rating=0;

		while (n>0) {
			if (_tags[top_tags[n-1]].rating>=n) {
				dream.rating=n;
				
				if (!(n in _authors[dream.author].rated_count))
					_authors[dream.author].rated_count[n]=0;

				_authors[dream.author].rated_count[n]++;

				if (_authors[dream.author].rated_count[n]>=n
						&& _authors[dream.author].rating<n)
					_authors[dream.author].rating=n;
				break;
			}

			n--;
		}
	});

	// compute group rating
	let top_authors=Object.keys(_authors).sort((a, b) => {
		return _authors[b].rating-_authors[a].rating;
	});

	for (let n=top_authors.length; n>=0; n--)
		if (_authors[top_authors[n-1]].rating>=n) {
			_group_rating=n;
			break;
		}
}


function get_last_dream_date() {
	let date;
	db.forEach(dream => {
		if (date===undefined || (dream.date && date<dream.date))
			date=dream.date;
	});

	if (date===undefined)
		return;

	return new Date(date);
}

var week_view;
class WeekView {
	// if date is unset then the date of the last dream is selected
	// date is either a string or a Date object
	constructor(date) {
		if (week_view!==undefined)
			throw Error('WeekView is a singletone');

		week_view=this;
		this.last_dream_date=get_last_dream_date();

		if (date===undefined)
			date=this.last_dream_date;

		if (typeof date==="string")
			date=new Date(date);

		this.el=document.createElement("div");
		this.el.className="week_view";

		this._date=date;
		this._reset();
	}

	_compute() {
		let shift=(this._date.getDay()+4)%7;

		this.wed=new Date(this._date.valueOf()-shift*24*60*60*1000);
		this.isowed=this.wed.toISOString().substr(0, 10);
		this.tue=new Date(this._date.valueOf()-(shift-6)*24*60*60*1000);
		this.isotue=this.tue.toISOString().substr(0, 10);
		this.week_no=Math.floor((this._date.valueOf()
			-new Date(BEGIN).valueOf()) /1000/60/60/24/7 + 1);

		this._wdreams_count=0;
		this._authors_this_week={};
		this._tags_this_week={};
		this._wdreams=[];
		db.forEach(dream => {
			if (dream.date && dream.date>=this.isowed
					&& dream.date<=this.isotue) {
				this._wdreams_count++;
				this._authors_this_week[dream.author]=1+
					(this._authors_this_week[dream.author] || 0);
				if (dream.tags) dream.tags.forEach(tag => {
					this._tags_this_week[tag] = 1+
					(this._tags_this_week[tag] || 0);
				});
				this._wdreams.push(JSON.parse(JSON.stringify(dream)));
			}
		});

		// compute week rating: n dreams with n shared tags
		this._week_clique=compute_clique(this._wdreams);
		this._week_rating=this._week_clique.length;
	}
	_create_prev_btn() {
		let btn=document.createElement("button");
		btn.innerText="<<";
		btn.onclick=()=>{
			this._date=new Date(this._date.valueOf()-7*24*60*60*1000);
			this._reset();
		};

		let week_no=Math.floor((this._date.valueOf()
			-new Date(BEGIN).valueOf()) /1000/60/60/24/7 + 1);

		if (week_no===1)
			btn.disabled=true;
		return btn;
	}

	_create_caption() {
		let el=document.createElement("span");
		el.className="week_caption";
		el.innerHTML=`
			Неделя № ${this.week_no}
			(${this.wed.toISOString().substr(0, 10)} &ndash;
			${this.tue.toISOString().substr(0, 10)})
		`;
		el.onclick=()=>{
			this._reset();
		};
		return el;
	}

	_create_next_btn() {
		let btn=document.createElement("button");
		btn.innerText=">>";
		btn.onclick=()=>{
			this._date=new Date(this._date.valueOf()+7*24*60*60*1000);
			this._reset();
		};

		if (this.last_dream_date<=this.tue)
			btn.disabled=true;

		return btn;
	}

	_create_stats() {
		let stats=document.createElement("div");

		// TODO authors list -> top
		// TODO authors -> link
		// TODO sort authors
		let sorted_authors=Object.keys(this._authors_this_week).sort(
			(a, b) => this._authors_this_week[b]
				- this._authors_this_week[a]);

		// TODO tags list -> top
		// TODO tag -> link

		let sorted_tags=Object.keys(this._tags_this_week).sort((a, b) => {
			return this._tags_this_week[b] - this._tags_this_week[a];
		});

		let tags=[];

		for (let i=0; i<MAX_TAGS && i<sorted_tags.length; i++) {
			let tag=sorted_tags[i];
			let tagc=this._tags_this_week[tag];
			tags.push(`<span class="tags_count">#${tag}</span> (${tagc})`);
		}

		let tags_suffix='';
		if (sorted_tags.length > MAX_TAGS)
			tags_suffix='...';

		let tags_count=Object.keys(this._tags_this_week).length;

		let authorc=get_authors_count();
		let tagc=get_tags_count();

		stats.appendChild(_bold("Общая статистика: "));
		stats.appendChild(_span("" + db.length, "link", show_all_dreams));
		stats.appendChild(_t(', '));
		stats.appendChild(_span('@'+authorc, "link author_banner", show_all_authors));
		stats.appendChild(_t(', '));
		stats.appendChild(_span('#'+tagc, "link tag_banner", show_all_tags));
		stats.appendChild(_t('. '));
		stats.appendChild(_span("Графики", "link", show_global_stats));
		stats.appendChild(_t('.'));

		stats.appendChild(_br());
		let wnum=this.week_no-1;
		wnum=wnum<10? '0'+wnum: ''+wnum;
		stats.appendChild(get_img(`data/generated/wcloud_${wnum}.png`));
		stats.appendChild(_br());

		stats.appendChild(_bold(
			`Рейтинг группы: ${_group_rating}`));
		stats.appendChild(_br());
		stats.appendChild(_bold(
			`Рейтинг недели: ${this._week_rating}`));
		stats.appendChild(_br());
		stats.appendChild(_bold(
			`Снов за неделю: ${this._wdreams_count}`));
		stats.appendChild(_br());

		stats.appendChild(_bold(
			`Авторов за неделю (${sorted_authors.length}): `));

		for (let i=0; i<sorted_authors.length; i++) {
			let author=sorted_authors[i];
			if (i>0)
				stats.appendChild(_t(", "));
			stats.appendChild(author_banner(author));
			stats.appendChild(_t(' '));
			stats.appendChild(_span(`(${this._authors_this_week[author]})`,
					"link", ()=>{
				clear(week_view.el);

				let back_div=document.createElement("div");
				back_div.appendChild(_button("Назад", ()=>{
					week_view.el.removeChild(back_div);
					week_view.week_header.style.display="block";
					week_view._reset();
				}));
				back_div.appendChild(_t(` Все сны ${author} на ${this.week_no} `
					+ `неделе (${this.isowed} – ${this.isotue})`));
				week_view.el.appendChild(back_div);

				week_view._tape=new DreamsTape(dream => dream.author===author
					&& dream.date >= this.isowed
					&& dream.date <= this.isotue);

				week_view.el.appendChild(week_view._tape.el);
				scrollTo(0, 0);
			}));
		}

		stats.appendChild(_br());

		stats.appendChild(_bold(`Теги за неделю (`));
		stats.appendChild(_span(`${Object.keys(this._tags_this_week).length}`,
				"link", ()=>{
			clear(week_view.el);

			let back_div=document.createElement("div");
			back_div.appendChild(_button("Назад", ()=>{
				week_view.el.removeChild(back_div);
				week_view.week_header.style.display="block";
				week_view._reset();
			}));
			back_div.appendChild(_t(` Все теги на ${this.week_no} `
				+ `неделе (${this.isowed} – ${this.isotue})`));
			week_view.el.appendChild(back_div);

			week_view.el.appendChild(_span(`Теги за всё время: ${Object.keys(_tags).length}`,
					"link", ()=>{
				show_all_tags();
			}));
			week_view.el.appendChild(_br());
			for (let i=0; i<sorted_tags.length; i++) {
				let tag=sorted_tags[i];
				week_view.el.appendChild(tag_banner(tag));
				week_view.el.appendChild(_t(' ('));
				week_view.el.appendChild(_span(`${this._tags_this_week[tag]}`,
						"link", ()=>{
					clear(week_view.el);

					let back_div=document.createElement("div");
					back_div.appendChild(_button("Назад", ()=>{
						week_view.el.removeChild(back_div);
						week_view.week_header.style.display="block";
						week_view._reset();
					}));
					back_div.appendChild(_t(` Все сны с тегом ${tag} на ${this.week_no} `
						+ `неделе (${this.isowed} – ${this.isotue})`));
					week_view.el.appendChild(back_div);

					week_view._tape=new DreamsTape(dream => dream.tags 
						&& dream.tags.indexOf(tag)!==-1
						&& dream.date >= this.isowed
						&& dream.date <= this.isotue);

					week_view.el.appendChild(week_view._tape.el);
					scrollTo(0, 0);
				}));
				week_view.el.appendChild(_t(', за всё время: '));
				week_view.el.appendChild(_span(`${_tags[tag].count}`,
						"link", ()=>{
					clear(week_view.el);

					let back_div=document.createElement("div");
					back_div.appendChild(_button("Назад", ()=>{
						week_view.el.removeChild(back_div);
						week_view.week_header.style.display="block";
						week_view._reset();
					}));
					back_div.appendChild(_t(` Все сны с тегом ${tag}`));
					week_view.el.appendChild(back_div);

					week_view._tape=new DreamsTape(dream => dream.tags 
						&& dream.tags.indexOf(tag)!==-1);

					week_view.el.appendChild(week_view._tape.el);
					scrollTo(0, 0);
				}));
				week_view.el.appendChild(_t(')'));
				week_view.el.appendChild(_br());
			}
			scrollTo(0, 0);
		}));
		stats.appendChild(_bold('): '));

		for (let i=0; i<sorted_tags.length && i<MAX_TAGS; i++) {
			let tag=sorted_tags[i];

			if (i>0)
				stats.appendChild(_t(", "));

			stats.appendChild(tag_banner(tag));
			stats.appendChild(_t(' '));
			stats.appendChild(_span(`(${this._tags_this_week[tag]})`,
					"link", ()=>{
				clear(week_view.el);

				let back_div=document.createElement("div");
				back_div.appendChild(_button("Назад", ()=>{
					week_view.el.removeChild(back_div);
					week_view.week_header.style.display="block";
					week_view._reset();
				}));
				back_div.appendChild(_t(` Все сны с тегом ${tag} на ${this.week_no} `
					+ `неделе (${this.isowed} – ${this.isotue})`));
				week_view.el.appendChild(back_div);

				week_view._tape=new DreamsTape(dream => dream.tags 
					&& dream.tags.indexOf(tag)!==-1
					&& dream.date >= this.isowed
					&& dream.date <= this.isotue);

				week_view.el.appendChild(week_view._tape.el);
				scrollTo(0, 0);
			}));
		}

		`
			<br/>
			<b>Тегов за неделю (${tags_count}):</b> ${tags.join(", ")}
			${tags_suffix}
			(тегов за всё время: ${get_tags_count()})
			<br/>
		`;

		return stats;
	}

	_create_day_banner(date, cb) {
		let el=document.createElement("div");
		let weekday=["вс", "пн", "вт", "ср", "чт", "пт", "сб"]
			[date.getDay()];
		let month=date.getMonth()+1;
		let short_date=date.getDate()+'.'
			+ (month<10? '0': '') + month;
		let authors={}, dreamsc=0, tags={};
		
		db.forEach(dream => {
			if (dream.date != date.toISOString().substr(0, 10))
				return;
			dreamsc++;
			authors[dream.author]=true;
			dream.tags.forEach(tag => {
				tags[tag]=true;
			});
		});
		let authorsc=Object.keys(authors).length;
		let tagsc=Object.keys(tags).length;
		el.innerHTML=`
			<span class="date_banner">${weekday} ${short_date}</span> <br/>
			${dreamsc} 
			<span class="author_count">@${authorsc}</span>
			<span class="tags_count">#${tagsc}</span>
		`;
		el.className="day_banner";

		el.onclick=()=>{
			this.el.removeChild(this._tape.el);

			this._tape=new DreamsTape(dream => {
				return dream.date == date.toISOString().substr(0, 10);
			});

			this.el.appendChild(this._tape.el);
			scrollTo(0, 0);

			if (cb) cb();
			el.className="day_banner_active";
		};

		return el;
	}

	_create_days() {
		let stripe=document.createElement("table");
		let tr=document.createElement("tr");
		stripe.appendChild(tr);
		for (let i=0; i<7; i++) {
			let date=new Date(this.wed.valueOf() + i*24*60*60*1000);
			let td=document.createElement("td");
			tr.appendChild(td);
			td.appendChild(this._create_day_banner(date, ()=>{
				tr.querySelectorAll('.day_banner_active').forEach(el => {
					el.className='day_banner';
				});
			}));
		}

		return stripe;
	}

	_reset() {
		this._compute();
		clear(this.el);
		this.week_header=document.createElement("div");
		this.week_header.appendChild(this._create_prev_btn());
		this.week_header.appendChild(this._create_caption());
		this.week_header.appendChild(this._create_next_btn());
		this.week_header.appendChild(document.createElement("br"));
		this.week_header.appendChild(this._create_stats());
		this.week_header.appendChild(this._create_days());
		this.el.appendChild(this.week_header);

		this._tape=new DreamsTape(dream => {
			return dream.date >= this.isowed
				&& dream.date <= this.isotue;
		});
		this.el.appendChild(this._tape.el);
		scrollTo(0, 0);
	}

}

function _button(caption, onclick) {
	let btn=document.createElement("button");
	btn.innerText=caption;
	if (onclick!==undefined)
		btn.onclick=onclick;
	return btn;
}

function _t(text) { return document.createTextNode(text); }

function _bold(text) {
	let b=document.createElement("b");
	b.innerText=text;
	return b;
}

function _span(text, cls, onclick) {
	let span=document.createElement("span");

	span.innerText=text;

	if (cls!==undefined)
		span.className=cls;

	if (onclick!==undefined)
		span.onclick=onclick;

	return span;
}

function _br() {
	return document.createElement("br");
}

class DreamsTape {
	constructor(filter) {
		if (filter===undefined)
			filter=()=>true;

		this.el=document.createElement("div");

		this._filter=filter;

		this._reset();
	}

	_reset() {
		clear(this.el);

		let dreams=[];

		db.forEach(dream => {
			if (this._filter(dream))
				dreams.push(dream);
		});

		dreams=dreams.sort((a, b)=> {
			if (a.date < b.date)
				return -1;
			else if (a.date > b.date)
				return 1;
			else
				return 0;
		});

		dreams.forEach(dream => {
			this.el.appendChild(new Dream(dream).el);
		});

		this._dreams=dreams;
	}
}

class Dream {
	constructor(dream) {
		this._dream=dream;

		this.el=document.createElement("div");
		this.el.className="dream";
		this._reset();
	}

	_reset() {
		clear(this.el);

		this.el.appendChild(author_banner(this._dream.author));
		this.el.appendChild(document.createTextNode(' '));
		this.el.appendChild(date_banner(this._dream.date));
		this.el.appendChild(document.createTextNode(' '));
		this.el.appendChild(_t(`[${this._dream.rating}]`));
		let a=document.createElement('a');
		let pid=this._dream.pid;
		a.setAttribute("target", "blank");
		a.setAttribute("href", `https://dreamhackers.eu/viewtopic.php?p=${pid}#p${pid}`);
		a.innerHTML=pid;
		a.className='graylink';
		this.el.appendChild(a);
		this.el.appendChild(document.createElement("br"));
		
		let dream_html=document.createElement("div");
		dream_html.className="dream_html";
		dream_html.innerHTML=this._dream.html;

		this.el.appendChild(dream_html);

		let tags=document.createElement("div");
		if (this._dream.tags) this._dream.tags.forEach(tag => {
			if (tags.children.length>0)
				tags.appendChild(document.createTextNode(' '));
			tags.appendChild(tag_banner(tag));
		});
		this.el.appendChild(tags);
	}
}

function add_days(date, days) {
	return new Date(date.valueOf() + days*1000*60*60*24);
}

function dreams_graph(canvas, dreams) {
	let el=canvas;

	let min_date;

	for (let i in dreams) {
		let dream=dreams[i];
		if (!min_date || min_date>dream.date)
			min_date=dream.date;
	}
	min_date='2021-01-20';
	min_date=new Date(min_date)

	let days=[];
	let legend=[];
	for (let i in dreams) {
		let dream=dreams[i];
		let d=(new Date(dream.date)-min_date)/1000/60/60/24;
		while (days.length<=d) {
			//legend.push(days.length);
			let cur=add_days(min_date, days.length);
			let l=cur.toISOString()
				.substr(8, 2);

			if (l==='01' || days.length===0) {
				l+='.' + cur.toISOString().substr(5, 2);
			}
			days.push(0);
			legend.push(l);
		}
		days[d]++;
	}

	let max_date=get_last_dream_date();

	while (true) {
		let cur=add_days(min_date, days.length);
		if (cur>max_date)
			break;

		let l=cur.toISOString().substr(8, 2);
		if (l==='01' || days.length===0)
			l+='.' + cur.toISOString().substr(5, 2);

		days.push(0);
		legend.push(l);
	}

	let chart=new Graph({
		data: days,
		target: el,
    // Internal padding for the canvas
    paddingBottom:  0, 
    paddingLeft:    0,
    paddingRight:   0,
    paddingTop:     30,

    // Canvas background
    background:     "#FFFFFFFF",

    // Show circle on the last data
    showCircle:     false,
    circle:         "#55AA77",
    circleSize:     4,

    // Show the line at zero
    centerZero:     false,
    showZeroLine:   true,
    zeroLineColor:  "#EEE",

    // Color and whidth of the line chart
    lineColor:      "#8888EE",
    lineWidth:      3,

    // Show the bounds
    showBounds:     true,
    bounds:         "#8888EE",
    boundsHeight:   14,
    boundsFont:     "Arial",

    // Show the legend
    showLegend:     true,
    legend:         legend,
    legendHeight:   14,
    legendFont:     "Arial",
	});
}

function author_banner(author) {
	let el=document.createElement("span");
	el.appendChild(_span("@", "author_banner_at"));
	el.appendChild(_span(author, `author_banner author_banner_level_${_authors[author].rating}`, ()=>{
		clear(week_view.el);

		let back_div=document.createElement("div");
		back_div.appendChild(_button("Назад", ()=>{
			week_view.week_header.style.display="block";
			week_view._reset();
		}));
		back_div.appendChild(_t(` Все сны ${author}`));
		week_view.el.appendChild(back_div);

		week_view._tape=new DreamsTape(dream => dream.author===author);

		week_view.el.appendChild(_bold(`Рейтинг юзера: ${_authors[
			author].rating}`));
		week_view.el.appendChild(_br());

		let dreamc=week_view._tape._dreams.length;
		week_view.el.appendChild(_t(`Всего снов: ${dreamc}`));
		week_view.el.appendChild(_br());

		let canvas=document.createElement("canvas");
		week_view.el.appendChild(canvas);
		dreams_graph(canvas, week_view._tape._dreams);

		week_view.el.appendChild(week_view._tape.el);
		scrollTo(0, 0);
	}));
	return el;
}

function date_banner(date) {
	let el=document.createElement("span");
	el.className="date_banner";
	el.innerHTML=date;
	el.onclick=()=>{
		clear(week_view.el);

		let back_div=document.createElement("div");
		back_div.appendChild(_button("Назад", ()=>{
			week_view._reset();
		}));
		back_div.appendChild(_t(` Все сны ${date}`));
		week_view.el.appendChild(back_div);

		week_view._tape=new DreamsTape(dream => dream.date===date);

		week_view.el.appendChild(week_view._tape.el);
		scrollTo(0, 0);
	}
	return el;
}

function tag_banner(tag) {
	let el=document.createElement("span");
	el.className=`tag_banner tag_level_${_tags[tag].rating}`;
	el.innerHTML='#'+tag;
	el.onclick=()=>{
		clear(week_view.el);

		let back_div=document.createElement("div");
		back_div.appendChild(_button("Назад", ()=>{
			week_view.el.removeChild(back_div);
			week_view.week_header.style.display="block";
			week_view._reset();
		}));
		back_div.appendChild(_t(` Все сны с тегом ${tag}`));
		week_view.el.appendChild(back_div);

		week_view.el.appendChild(_bold(`Рейтинг тега: ${
			_tags[tag].rating}`));
		week_view.el.appendChild(_br());

		week_view._tape=new DreamsTape(dream => dream.tags 
			&& dream.tags.indexOf(tag)!==-1);

		week_view.el.appendChild(week_view._tape.el);
		scrollTo(0, 0);
	}
	return el;
}

var gtags;

function main() {
	document.body.appendChild(new WeekView().el);
}

function show_week(n) {
	// Week 1 is Jan 20 .. Jan 26

}
